package jPicUsb;

/**
 * Interface with Microchip Usb API<br>
 * Requires jpicusb.dll on the same folder of running program<br>
 * Author: Geronimo Oñativia<br>
 * Version: 1.1.1<br>
 * Date: 30 de Enero de 2009
 * @author Geronimo
 */
public class iface {
    private static final Exception exFileNotFound = new Exception("Não encontrado jpicusb.dll em "+System.getProperty("user.dir") + System.getProperty("file.separator"));
    private static final Exception exSecurity = new Exception(" Exceção de segurança no carregamento do jpicusb.dll");
    private static final Exception exAlreadyLoaded = new Exception("DLL ja carregada");
    private static String ifaceLoaded = "Interface jPicUSB 1.1.1 Carregada";

    private static String pVID_PID = "vid_04d8&pid_000b";
    private static int instance = 0;
    private static int libLoaded = 0;
    /**
     * Called function failed. Useful on functions that can return zero.
     */
    public static int FUNCTION_CALL_FAIL = -1;
    /**
     * Called function failed. If function can return zero, use FUNCTION_CALL_FAIL instead.
     */
    public static int MPUSB_FAIL = 0;
    /**
     * Called function succeeded.
     */
    public static int MPUSB_SUCCESS = 1;
    /**
     * Direction of the endpoint. Use this to Write.
     */
    public static int MP_WRITE = 0;
    /**
     * Direction of the endpoint. Use this to Read.
     */
    public static int MP_READ = 1;
    /**
     * Max number of attached devices.
     */
    public static int MAX_NUM_MPUSB_DEV = 127;

    /**
     * Get mpusbapi revision level
     * @return String representing the version
     */
    public static native String GetAPIVersion();

    /**
     * Returns the number of devices with matching VID & PID.
     * @param pVID_PID A string containing the VID & PID value of the target device. The format is "vid_xxxx&pid_yyyy".
     * @return Number of USB device with matching VID & PID.
     */
    public static native int GetDeviceCount(String pVID_PID);

    /**
     * Opens an endpoint pipe with matching VID & PID.
     * @param instance An instance number of the device to open. 
     * @param pVID_PID A string containing the VID & PID value of the target device.  The format is "vid_xxxx&pid_yyyy".
     * @param pEP A string of the endpoint number on the target endpoint to open. The format is "\\MCHP_EPz". Where z is the endpoint number in decimal.
     * @param dwDir Specifies the direction of the endpoint. Use MP_READ for Read. Use MP_WRITE for Write.
     * @param dwReserved Future Use.
     * @return Returns a long value representing the handle to the endpoint pipe with matching VID & PID.
     */
    public static native long Open(int instance, String pVID_PID, String pEP, int dwDir, int dwReserved);

    /**
     * Opens an endpoint pipe with matching handle.
     * @param handle long value representing the handle to an endpoint pipe opened with Open.
     * @return True on sucess, False on fail.
     */
    public static native boolean Close(long handle);

    /**
     * Writes data to an opened endpoint pipe.
     * @param handle Long value representing the handle to an endpoint pipe opened with Open.
     * @param pData Byte array containing the data to be sent.
     * @param dwLen Amount of bytes to be sent.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Return the number of bytes written by this function call.
     */
    public static native long Write(long handle, byte[] pData, int dwLen, long dwMilliseconds);

    /**
     * Open a connection with matching VID & PID, writes data, and closes the opened endpoint pipe.
     * @param pVID_PID A string containing the VID & PID value of the target device.  The format is "vid_xxxx&pid_yyyy".
     * @param instance An instance number of the device to open.
     * @param pData Byte array containing the data to be sent.
     * @param dwLen Amount of bytes to be sent.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Return the number of bytes written by this function call.
     */
    public static native long Write(String pVID_PID, int instance, byte[] pData, int dwLen, long dwMilliseconds);

    /**
     * Reads data from an opened endpoint pipe.
     * @param handle Long value representing the handle to an endpoint pipe opened with Open.
     * @param dwLen Amount of bytes to be read.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Byte array containing the read data.
     */
    public static native byte[] Read(long handle, int dwLen, long dwMilliseconds);

    /**
     * Open a connection with matching VID & PID, reads data, and closes the opened endpoint pipe.
     * @param pVID_PID A string containing the VID & PID value of the target device.  The format is "vid_xxxx&pid_yyyy".
     * @param instance An instance number of the device to open.
     * @param dwLen Amount of bytes to be read.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Byte array containing the read data.
     */
    public static native byte[] Read(String pVID_PID, int instance, int dwLen, long dwMilliseconds);

    /**
     * Open two connections with matching VID & PID, wtite data, then reads data, and closes the opened endpoint pipes.
     * @param pVID_PID A string containing the VID & PID value of the target device.  The format is "vid_xxxx&pid_yyyy".
     * @param instance An instance number of the device to open.
     * @param pData Byte array containing the data to be sent.
     * @param dwLenWrite Amount of bytes to be sent.
     * @param dwLenRead Amount of bytes to be read.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Byte array containing the read data.
     */
    public static native byte[] WriteRead(String pVID_PID, int instance, byte[] pData, int dwLenWrite, int dwLenRead, long dwMilliseconds);

    /**
     * Quick Write<br>
     * Open a connection with DEFAULT VID & PID, writes data, and closes the opened endpoint pipe.
     * @param pData Byte array containing the data to be sent.
     * @param dwLen Amount of bytes to be sent.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Return the number of bytes written by this function call.
     */
    public static long QWrite(byte[] pData, int dwLen, long dwMilliseconds) {
        return Write(iface.pVID_PID, iface.instance, pData, dwLen, dwMilliseconds);
    }

    /**
     * Quick Read<br>
     * Open a connection with DEFAULT VID & PID, reads data, and closes the opened endpoint pipe.
     * @param dwLen Amount of bytes to be read.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Byte array containing the read data.
     */
    public static byte[] QRead(int dwLen, long dwMilliseconds) {
        return Read(iface.pVID_PID, iface.instance, dwLen, dwMilliseconds);
    }

    /**
     * Quick Write and Read<br>
     * Open two connections with DEFAULT VID & PID, wtite data, then reads data, and closes the opened endpoint pipes.
     * @param pData Byte array containing the data to be sent.
     * @param dwLenWrite Amount of bytes to be sent.
     * @param dwLenRead Amount of bytes to be read.
     * @param dwMilliseconds Timeout in milliseconds.
     * @return Byte array containing the read data.
     */
    public static byte[] QWriteRead(byte[] pData, int dwLenWrite, int dwLenRead, long dwMilliseconds) {
        return WriteRead(iface.pVID_PID, iface.instance, pData, dwLenWrite, dwLenRead, dwMilliseconds);
    }

    /**
     * Sets DEFAULT VID & PID used by functions wich don't receive a handle or VID & PID.
     * @param s A string containing the VID & PID value of the target device.  The format is "vid_xxxx&pid_yyyy".
     */
    public static void set_vidpid(String s) {
        iface.pVID_PID = s;
    }

    /**
     * Sets DEFAULT instance used by functions wich don't receive a handle or VID & PID.
     * @param i An instance number of the device.
     */
    public static void set_instance(int i) {
        iface.instance = i;
    }

    /**
     * Dummy constructor. All methods of this class are static. Its not necessary to create an instance of this class to use its methods.
     */
    public iface() {
        return;
    }

    /**
     * Loads the interface jpicusb.dll
     * Call this function only once.
     */
    public static void load() throws Exception {
        System.out.println(ifaceLoaded);
        if (iface.libLoaded == 1) {
            throw exAlreadyLoaded;
        }
        try{
            System.load(System.getProperty("user.dir") + System.getProperty("file.separator") + System.mapLibraryName("jpicusb"));
            iface.libLoaded = 1;
        }catch(UnsatisfiedLinkError e){
            throw exFileNotFound;
        }catch(SecurityException e){
            throw exSecurity;
        }
    }
}