//Autor: Marco Biscaro
//marcobiscaro2112@gmail.com
//http://www.guj.com.br/java/149034-duvidas-ao-fazer-uma-consulta
//http://www.guj.com.br/posts/downloadAttach/4732.java
//Código modificado por Esmael Dias Prado.

package tabela.servos;

import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;
import tabela.tempos.Tempo;


public class ServoTableModel extends AbstractTableModel {

	/**
	 * Essa lista armazenará os objetos do tipo {@link Servo} atualmente
	 * exibidos na tablela.
	 */
	private List<Servo> clientes;

	// ==============================================================
	// Construtores.
	// ==============================================================

	/**
	 * Constutor que simplesmente instancia o nosso {@link List} que usaremos
	 * para guardar os valores.
	 */
	public ServoTableModel() {
		// no construtor, instanciamos o List
		clientes = new ArrayList<Servo>();
	}

	/**
	 * Criamos um construtor de conveniência para já popular a lista.
	 * 
	 * @param lista
	 *            a lista de clientes a ser adicionada.
	 */
	public ServoTableModel(List<Servo> lista) {
		this();
		clientes.addAll(lista);
	}

	// ==============================================================
	// Métodos implementados.
	// ==============================================================

	@Override
	public Class<?> getColumnClass(int coluna) {
		// todas as colunas representam uma String
		return String.class;
	}

	@Override
	public int getColumnCount() {
		// esse método deve retornar o número de colunas. No caso, 3 (uma para o
		// nome, uma para o sobrenome e uma para o apelido).
		return 3;
	}

	@Override
	public String getColumnName(int coluna) {
		// vamos retornar o nome de cada coluna
		switch (coluna) {
		case 0:
			return "Motor"; // o nome da primeira coluna
		case 1:
			return "Valor Inicial"; // o nome da segunda
		case 2:
			return "Valor Final"; // e o da terceira
		default:
			return ""; // isso nunca deve ocorrer, pois temos só 3 colunas
		}
	}

	@Override
	public int getRowCount() {
		// retorna o número de linhas, ou seja, a quantidade de entradas na
		// nossa lista.
		return clientes.size();
	}

	@Override
	public Object getValueAt(int linha, int coluna) {
		// vai retornar o valor de determinada célula. A linha representa a
		// posição do Servo na nossa lista e a coluna vai ser: 1 - nome, 2 -
		// sobrenome e 3 - apelido
		// primeiro vamos pegar o Servo da linha
		Servo c = clientes.get(linha);
		// façamos um switch
		switch (coluna) {
		case 0:
			return c.getNome(); // retornamos o nome
		case 1:
			return c.getSobrenome(); // retornamos o sobrenome
		case 2:
			return c.getApelido(); // e o apelido
		default:
			return null; // isso nunca deve ocorrer, pois temos só 3 colunas
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		// nesse caso, todas as células são editáveis
		return true;
	}

	@Override
	public void setValueAt(Object valor, int linha, int coluna) {
		// aqui devemos atualizar o valor de nossos Clientes
		// vemos em qual linha ele está
		Servo c = clientes.get(linha);
		// e vemos o que será atualizado
		switch (coluna) {
		case 0:
			c.setNome(valor.toString()); // mudamos o nome
			break;
		case 1:
			c.setSobrenome(valor.toString()); // mudamos o sobrenome
			break;
		case 2:
			c.setApelido(valor.toString()); // e o apelido
			break;
		}
		// é importante notificar os listeners a cada alteração
		fireTableDataChanged();
	}
        
        
        public void setLinhaServo(int linha, int s1, int s2) {
		Servo c = clientes.get(linha);

                    c.setSobrenome(Integer.toString(s1)); // mudamos o sobre
                    c.setApelido(Integer.toString(s2)); // e o apelido

		// é importante notificar os listeners a cada alteração
		fireTableDataChanged();
	} 


                /**
        *
        * @param linha
        * @param coluna
        * @return
        */
        public String getCelula(int linha, int coluna) {
		// vai retornar o valor de determinada célula. A linha representa a
		// posição do Tempo na nossa lista e a coluna vai ser: 1 - nome, 2 -
		// sobrenome e 3 - apelido
		// primeiro vamos pegar o Tempo da linha
		Servo c = clientes.get(linha);
		// façamos um switch
		switch (coluna) {
		case 0:
			return c.getNome();
                case 1:
			return c.getSobrenome();
		case 2:
			return c.getApelido();
		default:
			return null; // isso nunca deve ocorrer, pois temos só 3 colunas
		}
	}

	/**
	 * Adiciona um novo {@link Servo} e notifica os listeners que um novo
	 * registro foi adicionado.
	 */
	public void adiciona(Servo c) {
		clientes.add(c);
		// informamos os listeners que a linha (size - 1) foi adicionada
		fireTableRowsInserted(clientes.size() - 1, clientes.size() - 1);
	}

	/**
	 * Similar ao {@link #adiciona(Servo)}, porém para remover. Recebe o
	 * índice do Servo a ser removido. Se não souber o índice, use o método
	 * {@link #getIndice(Servo)} antes.
	 */
	public void remove(int indice) {
		clientes.remove(indice);
		fireTableRowsDeleted(indice, indice);
                ServoLinhas.i_MenosMenos();
	}

	/**
	 * Retorna o índice de determinado Servo.
	 */
	public int getIndice(Servo c) {
		return clientes.indexOf(c);
	}

	/**
	 * Adiciona todos os clientes na lista à este modelo.
	 */
	public void adicionaLista(List<Servo> lista) {
		int i = clientes.size();
		clientes.addAll(lista);
		fireTableRowsInserted(i, i + lista.size());
	}

	/**
	 * Esvazia a lista.
	 */
	public void limpaLista() {
		int i = clientes.size();
		clientes.clear();
		fireTableRowsDeleted(0, i - 1);
	}

}
