
///////////////////////////////////////////////////////////////////////////
////   Library for a 24LC512 serial EEPROM                             ////
////                                                                   ////
////   init_ext_eeprom();    Call before the other functions are used  ////
////                                                                   ////
////   write_ext_eeprom(a, d);  Write the byte d to the address a      ////
////                                                                   ////
////   d = read_ext_eeprom(a);   Read the byte d from the address a    ////
////                                                                   ////
////   The main program may define eeprom_sda                          ////
////   and eeprom_scl to override the defaults below.                  ////
////                                                                   ////
///////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,2003 Custom Computer Services           ////
//// This source code may only be used by licensed users of the CCS C  ////
//// compiler.  This source code may only be distributed to other      ////
//// licensed users of the CCS C compiler.  No other use, reproduction ////
//// or distribution is permitted without written permission.          ////
//// Derivative programs created using this software in object code    ////
//// form are not restricted in any way.                               ////
///////////////////////////////////////////////////////////////////////////

#ifndef EEPROM_SDA
#define EEPROM_SDA  PIN_C7
#define EEPROM_SCL  PIN_C6
#endif

//#use i2c(master, sda=EEPROM_SDA, scl=EEPROM_SCL,FAST=100000,stream=MEM)    //100 KHz
#use i2c(master, sda=EEPROM_SDA, scl=EEPROM_SCL,FAST=400000,RESTART_WDT,stream=MEM)  //400 KHz
//#use i2c(master, sda=EEPROM_SDA, scl=EEPROM_SCL,FAST=1000000,stream=MEM) //1000 KHz = 1MHz
//#use i2c(master, sda=EEPROM_SDA, scl=EEPROM_SCL,FAST=10000000,stream=MEM)//10000 KHz = 10MHz


/////////////////////////////////////////////////////////////////////////////////////////////
// I S 1 0 1 0 A2 A1 A0 R/W
//
//   |-----------|-------|---------------------------------|-----------------------|--------|
//   |  INICIO   |B.START|        CODIGO DE CONTROLE       |EDEREO DO DISPOSITIVO |  R/W   |
//   |-----------|-------|---------------------------------|-----------------------|--------|  
//   |  INICIO   | START |        CODIGO DE CONTROLE       |    A2      A1      A0 |ESCRITA |
//   |           |       |                                 |                       |        |
//   |           |       |                                 |                       |        |
//SDA|     F     | 0   0 |     1       0       1       0   |   0       0       0   |   0    |
//   |           |       |                                 |                       |        |
//SCK|     F     | 1   0 | 0   1   0   1   0   1   0   1   0   1   0   1   0   1   0   1   0|
//   |           |       |                                 |                       |        | 
//   |           |       |                                 |                       |        | 
//   |___________|       |  _______         _______        |                       |        | 
//SDA|  FFFFFFFFF|___0___|_|   1   |___0___|   1   |___0___|___0___|___0___|___0___|___0____| 
//||||||
//   |___________|___    |    ___     ___     ___     ___     ___     ___     ___     ___   |        
//SCK|  FFFFFFFFF| 1 |_0_|_0_| 1 |_0_| 1 |_0_| 1 |_0_| 1 |_0_| 1 |_0_| 1 |_0_| 1 |_0_| 1 |_0|
//||||
//   |           |       |                                 |                       |        | 
//---|-----------|-------|------------------------------------------------------------------|  
//   |           |       |                                 |                       |        | 
//   |     1     | 0   0 |     1       0       1       0       0       0       0       0    |                     
//   |___________|       |  _______         _______        |                       |        |
//SDA|           |_______|_|       |_______|       |________________________________________|                                          
//||||
//   | F   F   F | 0   0 | 1   1   1   0   1   1   1   0   0   0   0   0   0   0   0   0   0|
//---|-----------|-------|------------------------------------------------------------------|  
//   | F   F   F | 1   0 | 0   1   0   1   0   1   0   1   0   1   0   1   0   1   0   1   0|   
//   |___________|___    |    ___     ___     ___     ___     ___     ___     ___     ___   |  
//SCK|           |   |___|___|   |___|   |___|   |___|   |___|   |___|   |___|   |___|   |__|
//---|-----------|-------|---------------------------------| ----------------------|--------|

// Tamanho  - Tamanho    - tamanho Hex - ltimo End HEX
//  16 kbit -   2 KBytes - 800         - 007FF
//  32 kbit	-   4 KBytes - 1000	       - 00FFF
//  64 kbit	-   8 KBytes - 2000	       - 01FFF
// 128 kbit	-  16 KBytes - 4000	       - 03FFF
// 256 kbit	-  32 KBytes - 8000	       - 07FFF
// 512 kbit	-  64 KBytes - 10000	   - 0FFFF
//   1 Mbit	- 128 KBytes - 20000	   - 1FFFF
//   2 Mbit	- 256 kbytes - 40000	   - 3FFFF
//   4 Mbit	- 512 kbyte	 - 80000	   - 7FFFF


//Ativos - 0, 2 ,4

//Slave address.  A0=0,A1=0,A2=0        //Usado no relogio PCF8583
//1 0 1 0 A2 A1 A0 R/W                  
#define EEPROM_0_WRITE_ADDRESS 0xA0     //1010 000 0
#define EEPROM_0_READ_ADDRESS  0xA1     //1010 000 1 

//Mesmo do Relgio PCF 
//Slave address.  A0=1,A1=0,A2=1        ************
//1 0 1 0 A2 A1 A0 R/W
#define EEPROM_1_WRITE_ADDRESS 0xA2     //1010 001 0
#define EEPROM_1_READ_ADDRESS  0xA3     //1010 001 1

//Slave address.  A0=0,A1=1,A2=0        ************
//1 0 1 0 A2 A1 A0 R/W
#define EEPROM_2_WRITE_ADDRESS 0xA4     //1010 010 0
#define EEPROM_2_READ_ADDRESS  0xA5     //1010 010 1

//Slave address.  A0=0,A1=1,A2=1		************
//1 0 1 0 A2 A1 A0 R/W
#define EEPROM_3_WRITE_ADDRESS 0xA6     //1010 011 0
#define EEPROM_3_READ_ADDRESS  0xA7     //1010 011 1

//Slave address.  A0=1,A1=0,A2=0        
//1 0 1 0 A2 A1 A0 R/W
#define EEPROM_4_WRITE_ADDRESS 0xA8     //1010 100 0
#define EEPROM_4_READ_ADDRESS  0xA9     //1010 100 1

//Slave address.  A0=1,A1=0,A2=1
//1 0 1 0 A2 A1 A0 R/W
#define EEPROM_5_WRITE_ADDRESS 0xAA     //1010 101 0
#define EEPROM_5_READ_ADDRESS  0xAB     //1010 101 1

//Slave address.  A0=1,A1=1,A2=0
//1 0 1 0 A2 A1 A0 R/W
#define EEPROM_6_WRITE_ADDRESS 0xAC     //1010 110 0
#define EEPROM_6_READ_ADDRESS  0xAD     //1010 110 1

//Slave address.  A0=1,A1=1,A2=1
//1 0 1 0 A2 A1 A0 R/W
#define EEPROM_7_WRITE_ADDRESS 0xAE     //1010 111 0
#define EEPROM_7_READ_ADDRESS  0xAF     //1010 111 1


#define EEPROM_ADDRESS long int
#define EEPROM_SIZE   65535      //para 24LC512 = 65535, 24LC1025 = 131071

//**************************************************************************** 
void init_ext_eeprom()
{
   output_float(EEPROM_SCL);
   output_float(EEPROM_SDA);

}
//**************************************************************************** 

//****************************************************************************
//Escrita aleatia da memoria 
void write_ext_eeprom(long int address, BYTE data, int dispositivo)
{
   short int status;

   i2c_start();												//Inicia I2C
   i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1));  //Enderea o dispositivo correto
   i2c_write(address>>8);                                   //Escreve endereo alto 8 bits
   i2c_write(address);                                      //Escreve o endereo baixo 8 bits
   i2c_write(data);											//Escreve o dado 8 bits					
   i2c_stop();												//Finaliza o I2C

   //Testa se finalizou a gravao do dispositivo senao deverar esperar num delay de 5ms	
   i2c_start();
   status=i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1));
   while(status==1)
   {
      i2c_start();
      status=i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1));
   }
}
//**************************************************************************** 


//****************************************************************************
//Leitura aleatria da memoria  
BYTE read_ext_eeprom(long int address, int dispositivo) {

   BYTE data;
   
   i2c_start();
   i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1) );
   i2c_write(address>>8);
   i2c_write(address);
   i2c_start();
   i2c_write(EEPROM_0_READ_ADDRESS | (dispositivo << 1) );
   data=i2c_read(0);
   i2c_stop();
   return(data);
}
//**************************************************************************** 


//****************************************************************************
//Escrita sequencial, lembrado que as pginas tem que ser de 128 em 128 Bytes. at 65535
//   0 ----- 127
// 128 ----- 255
// 256 ----- 383
// 384 ----- 511
// 639 ----- 766 
//  ... at  65535
//
//65536 = Bytes
//512   = Pginas
void write_ext_eeprom_sequencial(long int address, BYTE data[], int tamanho_data, int dispositivo)
{
   short int status;
   int i; 

   i2c_start();												//Inicia I2C
   i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1));  //Enderea o dispositivo correto
   i2c_write(address>>8);                                   //Escreve endereo alto 8 bits
   i2c_write(address);                                      //Escreve o endereo baixo 8 bits
   
   for(i=0; i < tamanho_data; i++){
   	i2c_write(data[i]);										//Escreve o dado 8 bits					
   }
	
   i2c_stop();												//Finaliza o I2C

   //-----------------------------------------------------------------------------------
   //Testa se finalizou a gravao do dispositivo senao deveria ter num delay de 5ms
   i2c_start();
   status=i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1));
   while(status==1)
   {
      i2c_start();
      status=i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1));
   }
   //delay_ms(5);
}
//****************************************************************************

//****************************************************************************
//Leitura sequencial da memoria 
//BYTE data[]						 
void read_ext_eeprom_sequencial(long int address, BYTE data[], int tamanho_data, int dispositivo){
   	int i;	

   	i2c_start();												//Inicia I2C
   	i2c_write(EEPROM_0_WRITE_ADDRESS | (dispositivo << 1) ); 	//Enderea o dispositivo correto
   	i2c_write(address>>8);										//Escreve endereo alto 8 bits
   	i2c_write(address);											//Escreve o endereo baixo 8 bits
   	i2c_start();												//Escreve o endereo baixo 8 bits
   	i2c_write(EEPROM_0_READ_ADDRESS | (dispositivo << 1) );  	//Enderea o dispositivo correto
	for(i=0;i< tamanho_data-1;i++){			
   		data[i]=i2c_read();										//L o dado 8 bits em sequncia										
   	}
   	data[tamanho_data-1]=i2c_read(0);							//L o dado 8 bits e gera NoAck	
   	i2c_stop();													//Finaliza a leitura
   	//return(data);												//Retorna o array byte
}
//**************************************************************************** 