//SERB.c
//Software Embarcado Rob Bpede V01.0.7
//Autor: Esmael Dias Prado, 2015, zz9abi@gmail.com

#include <18F4550.h>	//Incluso das definies do microcontrolador utilizado.
#device adc=10			//Define a entrada analgia de 10bit's.	

//Configurao dos fuses interno do PIC18F4553.
#fuses HSPLL        //Cristal ou ressonador externo de alta frequncia maior que 4MHz juntamente 
                    //com o PLLDIV ativado, cristal utilizado 20MHz. 
#fuses PLL5         //Ativa o PLL com divisor por 5 na frequcnia do oscilador HSPLL, assim 20MHz/5 = 4MHz; 
					//o 4Mhz  padro para o PLL multiplicar a frequncia e gerar 96MHz.
					//Para melhor entendimento veja "Datasheet Clock Diagram".  
#fuses USBDIV       //Divide 96MHz/2 gerando 48Mhz necessrio, para o funcionaemnto da USB.
#fuses VREGEN       //Ativa a voltagem regulada da USB. 
#fuses CPUDIV1      //Divide 96MHz/2 para gerar 48MHz para a CPU (12 MIPS).
#fuses PUT          //Ativa o oscilador temporizador "start-up" (OST) "Power Up Timer",
                    //gera um atraso de 72ms para estabilizar o oscilador na energizao. 
#FUSES BORV46       //Reseta o processador se tenso cair a menos de 4.6V, "Brownout reset". 
#fuses MCLR			//Ativa o pino /MCLR-RC3 "Master Clear" (Pino para RESETAR a CPU).

#fuses NOWDT 		//Desativa o "Watch Dog Timer".
#fuses NOLVP		//Desativa a programao de baixa voltagem.
#fuses NODEBUG      //Desativa o Debug em modo ICD. 
#fuses NOFCMEN      //Desativa o monitor contra falha de Relgio (Relgio interno RTC no utilizado), 
                    //"Fail-safe clock monitor disabled". 
#fuses NOPROTECT    //Desativa a proteo de leitura de cdigo de programao.
#fuses NOCPD        //Desativa a proteo de leitura da EEPROM.
#fuses NOCPB        //Desativa a proteo de leitura do cdigo do bloco de inicializao, "Boot block". 
#fuses NOWRT        //Desativa a proteco de escrita na memria de programa. 
#fuses NOWRTD       //Desativa a proteo de escrita de dados na EEPROM. 
#fuses NOWRTB       //Desativa a proteo de escrita no bloco de inicializao, "Boot block". 
#fuses NOWRTC       //Desativa a proteo de escrita nos registradores. 
#fuses NOEBTRB      //Desativa a proteo do bloco de inicalizao "Boot block" em outras tabelas de leitura.
#fuses NOEBTR       //Desativa a proteo da memria em outras tabelas de leitura. 

#use delay(clock=48000000)

#include <stdio.h>   // para o printf(), atoi(), itoa();     
#include <stdlib.h>
#include "servo_init.h"

#include <AT24C512.h>
//#include <PCF8583_RTC.h>

//Tipos de comunicao selecionvel
const int USB_DIRETO = 0;
const int USB_SERIAL = 1;

//ABILITA O TIPO DE COMUNICAO PRETENDIDA 
const int ATIVA = USB_DIRETO;

int LIG_DESL_SERVOS = 0;
int LIG_DESL_SERVOS1 = 0;

//Mdia da leitura de tensao
int8 buffer_tensao[50];
int cont_media_tensao=0;

//****************************************************************************
#IF ATIVA == USB_DIRETO
   	  #include "pacote_USB_direto.h" //Se utilizar esta biblioteca no utilize a biblioteca pacote_USB_serial.h
#ENDIF

#IF ATIVA == USB_SERIAL
   	  #include "pacote_USB_serial.h" //Se utilizar esta biblioteca no utilize a biblioteca pacote_USB_direto.h
#ENDIF
//****************************************************************************

//****************************************************************************

//---------------------------------------------------------------------------
//Timer 1
#define TIMER1OFFSET 65534
//16 bit = 65536
// setup_timer_1(RTCC_INTERNAL|RTCC_DIV_1);  //48MHz/4/1 = 12MHz  = 0,08333us
//---------------------------------------------------------------------------

//Timer 3 									 //c = 1 ms
#define TIMER3OFFSET 53550

int8 x=0;						             //Conta varias interrupcoes CCP1 sem pulso
//boolean led1=0;
int16 recebe_tempo_t1=0;
int16 recebe_tempo_t3=0;
int16 cont_temp1=0;
int16 cont_temp2=0;

#include "servo.h"

//****************************************************************************
//Mandar p/ a varivel global a inicializao dos dados, para executar os cenrios
void inicia_valor_memoria(){
	int   i;
	int   cont=0;
	int8  dado[128];
	int32 tamanho_endereco;

	for(i=0; i<= 30; i++){
		endereco_memoria[i]=0;
	}	

	//Marcando a varivel Global com o valor do total de cenrios
	//Este valor sempre estar na poio zero e tem o tamanho de um Byte (0-255)
 	//BYTE read_ext_eeprom(long int address, int dispositivo)
	//No foi previsto que o total de enderecamento de cenrio ultrapasse o banco 1 de 
	//memoria pois seria incapaz de caber os cenrios nos demais 2 bancos de memoria.
	//E para economizar memoria est definido 128 Bytes de edereamentos
	//Totalizando um total de (128/3) - 1 = 41 cenrios possveis
	//Mais devido a limitaes de memoria do pic ser limitande uns 30 cenarios 
	 
	total_cenario = read_ext_eeprom(0, 1);	//funo retorna um byte, total de cenrios
	
	if(total_cenario!=0){
		tamanho_endereco = 3 * (total_cenario + 1);	

		//Leitura dos endereo de cada cenrio so composto de 3 Bytes p/ cada endereamento,
		//para isso ser buscado a partir do endereo (1 at ((3 x Byte) x (total_cenario + 1))
		//Ex:
		//End. 0x0000 = Total de cenrio
		//End. 0x0001 = Byte alto do endereo inicial do cenrio 1
		//End. 0x0002 = Byte do meio do endereo inicial do cenrio 1
		//End. 0x0003 = Byte mais baixo do endreo inicial do cenario 1
		//
		//End. 0x0004 = Byte alto do endereo final do cenrio 1
		//End. 0x0005 = Byte do meio do endereo final do cenrio 1
		//End. 0x0006 = Byte mais baixo do endreo final do cenario 1
		//
		//End. 0x0007 = Byte alto do endereo final do cenrio 2
		//End. 0x0008 = Byte do meio do endereo final do cenrio 2
		//End. 0x0009 = Byte mais baixo do endreo final do cenario 2
		//... assim por diante
		//read_ext_eeprom_sequencial(long int address, BYTE data[], int tamanho_data, int dispositivo)
		//Lendo a memria extern a a partir do endereo 1, no banco 1 de memoria
		read_ext_eeprom_sequencial(1, dado, tamanho_endereco, 1);

		// tamanho_endereco = (0 at tamanho_endereco-1)
		for (i=0;i<tamanho_endereco; i+=3){
			endereco_memoria[cont] = make32(dado[i],dado[i+1],dado[i+2]);
			cont++;
		}
	}
}
//****************************************************************************

//**************************************************************************** 
int16 executa_cenario_memoria_local(void){
	
	int16 tempo=1;
	//t   i;
	int   seleciona_men;
	int   tamanho_endereco=32;
	int8  dado[33];
	int1  flag = 0;
	int16 temp;

	//total_cenario
	if(contagem_endereco_mem==0){
		contagem_endereco_mem = endereco_memoria[0];
	}

	//Estouro de memoria >196.607
	if( contagem_endereco_mem <= 196607 ){		  //Se o endereo for menor ou igual a 196607 entra, endereo vlido.
		seleciona_men = 1;						  //Selecione o banco 1 de memria
		if(contagem_endereco_mem>65535){		  //Se for maior que entre (0 e 65535), ento banco 2 apartir de 65536;
			seleciona_men = 2;					  //Selecione o banco 2 de memria
			if(contagem_endereco_mem>131071){	  //Se for maior que entre (65536 e 131071 ), ento banco 3 apartir de 131072;
				seleciona_men = 3;				  //Selecione o banco 3 de memria
				if(contagem_endereco_mem>196607){ //Se for maior que entre (131071 e 196607), ento banco 4 apartir de 196608;
					seleciona_men = 1;  		  //Selecione o banco 1 de memria pois o banco 4 no exite. h um erro.
				}
			}
		}

		//read_ext_eeprom_sequencial(long int address, BYTE data[], int tamanho_data, int dispositivo)
		if( contagem_endereco_mem <= endereco_memoria[1] ){

			read_ext_eeprom_sequencial(contagem_endereco_mem, dado, tamanho_endereco, seleciona_men);
			contagem_endereco_mem = contagem_endereco_mem+32;

			tempo  = make16(dado[0],dado[1]);

			//Motor 1 = limite min 130 + 7900 = 8030,   Limite Max = 21100 + 7900 = 29000
			temp = make16(dado[2],dado[3]);	
			if(temp<130){temp=130;}
			if(temp>20920){temp=20920;}
			servo1 = temp+7900;

			//Motor 2 = limite min 130 + 7900 = 8030,   Limite Max = 20360 + 7900 = 28260
			temp = make16(dado[4],dado[5]);			
			if(temp<130){temp=130;}
			if(temp>20360){temp=20360;}	
			servo2 = temp+7900;

			//Motor 3 = limite min 130 + 7900 = 8030,   Limite Max = 20360 + 7900 = 28260
			temp = make16(dado[6],dado[7]);
			if(temp<130){temp=130;}
			if(temp>20360){temp=20360;}	
			servo3 = temp+7900;

			//Motor 4 = limite min 600 + 7900 = 8500,   Limite Max = 20900 + 7900 = 28800
			temp = make16(dado[8],dado[9]);
			if(temp<600){temp=600;}
			if(temp>20900){temp=20900;}
			servo4 = temp+7900;

			//Motor 5 = limite min 600 + 7900 = 8500,   Limite Max = 20900 + 7900 = 28800
			temp = make16(dado[10],dado[11]);
			if(temp<600){temp=600;}
			if(temp>20900){temp=20900;}
			servo5 = temp+7900;

			//Motor 6 = limite min 1127 + 7900 = 8500,   Limite Max = 18450 + 7900 = 26350
			temp = make16(dado[12],dado[13]);
			if(temp<1127){temp=1127;}
			if(temp>18450){temp=18450;}
			servo6 = temp+7900;

			//Motor 7 
			temp = make16(dado[14],dado[15]);
			servo7 = temp+7900;

			//Motor 8 
			temp = make16(dado[16],dado[17]);
			servo8 = temp+7900;

			//Motor 9 
			temp = make16(dado[18],dado[19]);
			servo9 = temp+7900;

			//Motor 10 
			temp = make16(dado[20],dado[21]);
			servo10 = temp+7900;

			//Motor 11 
			temp = make16(dado[22],dado[23]);
			servo11 = temp+7900;

			//Motor 12 
			temp = make16(dado[24],dado[25]);
			servo12 = temp+7900;

			//Motor 13 
			temp = make16(dado[26],dado[27]);
			servo13 = temp+7900;

			//Motor 14 
			temp = make16(dado[28],dado[29]);
			servo14 = temp+7900;

			//Motor 15 
			temp = make16(dado[30],dado[31]);
			servo15 = temp+7900;

		}else{
			if( contagem_endereco_mem-2 <= endereco_memoria[1] ){
				tempo=1000;
				contagem_endereco_mem+=2;
			}else{
				tempo=5;
				LIG_DESL_SERVOS=0;
				LIG_DESL_SERVOS1=0;
				contagem_endereco_mem=0;
			}		
		}

	}else{
		//estoutrou o limite da memoria
		//faz nada retorna tempo = 1;
	}
	return tempo*3;    //x3 original
}
//**************************************************************************** 

//****************************************************************************
void verifica_bateria(void){
	float ad1;
	int i;
	int16 media=0;

    set_adc_channel(1);
	delay_us(80);
	ad1=read_adc();
	ad1=0.2466258*ad1;
	
	if( cont_media_tensao >= 50){

		for(i=0;i < 50; i++){
			media = media + buffer_tensao[i];
		}
		media = media / 50;

		if( media < 70){          
			while(1){	
				output_high(pin_a2);
				delay_ms(500);
	  			output_low(pin_a2);
				delay_ms(500);
   			}
		}
		cont_media_tensao=0;
	}else{

		buffer_tensao[cont_media_tensao] = ad1;
		cont_media_tensao++;
	}	
}
//****************************************************************************

//****************************************************************************
void rotina_1_ms(void){

	cont_temp1++;
	cont_temp2++;

	if(LIG_DESL_SERVOS1==1){
		if(cont_temp1>=cont_geral1){

			led1 = !led1;
			output_bit(pin_a2,led1);
			cont_temp1=0;

			cont_geral1 = executa_cenario_memoria_local();
		}
	}

	//executa a cada 20ms
	if(cont_temp2>=cont_geral2){
      	verifica_bateria();
		cont_temp2=0;
		cont_geral2 = 20;
	}
}
//****************************************************************************

//****************************************************************************
void main(){
    int16 w=0;

   //Configura as porta analogicas Pino P19/RA0/AN0 e P20/RA1/AN1
   setup_ADC_ports (AN0_TO_AN1_ANALOG);
   setup_adc(ADC_CLOCK_INTERNAL );	

   setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);
   set_timer1(TIMER1OFFSET);
   setup_timer_0(RTCC_INTERNAL|RTCC_8_BIT|RTCC_DIV_64);
   set_timer0(TIMER3OFFSET);

   enable_interrupts(INT_TIMER1);
   enable_interrupts(INT_TIMER0);

   for(w=0;w<5;w++){
		output_high(pin_a2);
		//output_low(pin_a3);
		delay_ms(30);
	  	output_low(pin_a2);
	  	//output_high(pin_a3);
		delay_ms(180);
   }
   output_low(pin_a3);

   #IF ATIVA == USB_DIRETO
        //Inicializao da porta usb direta--------------------------------------------
   	    usb_init(); //inicializamos el USB
   		usb_task(); //Se encarga de mantener el  sentido de la comunicacin, llama a usb_detach() yusb_attach() cuando se necesita
		
		//Se o boto estiver apertado salte a enumerao da porata USB p/ Funcionar sem cabo.
		if(false!=input(pin_c1))			  //Verifique se o Boto foi precionado nivel '0'.
		{
   			usb_wait_for_enumeration(); 	  // Esperamos hasta que el PicUSB sea configurado por el host
		}else{
			delay_ms(2000);
		}
   		//-----------------------------------------------------------------------------
   #ENDIF

   #IF ATIVA == USB_SERIAL
   		//Inicializao da porta serial------------------------------------------------
   		usb_cdc_init();
   		usb_init();
   		//-----------------------------------------------------------------------------  
   #ENDIF
 
   enable_interrupts(global); // Habilitamos todas las interrupciones
   
   output_b(0);
   output_d(0);
   output_e(0);
   init_ext_eeprom();
   inicia_valor_memoria();	 
   LIG_DESL_SERVOS=0;
   LIG_DESL_SERVOS1=0;

 while(true)
 {
	#IF ATIVA == USB_DIRETO
		recebe_pacote_usb();
	#ENDIF
	
	#IF ATIVA == USB_SERIAL
		gets_usb_serial();
	#ENDIF

	if(false==input(pin_c1))			  //Verifique se o Boto foi precionado nivel '0'.
	{	
		inicia_valor_memoria();           //Inicializa o cabealho da memria total de cenrio e endereos

		LIG_DESL_SERVOS=1;                //Liga Servos
		LIG_DESL_SERVOS1=1;
		delay_us(200);		 			  //Tempo para Debouncing, para evitar rudos de chave. 
		while(false==input(pin_C1)){}     //Espere o boto ser solto.
	}
 }//while
}
//****************************************************************************

//****************************************************************************
#int_TIMER1
void TIMER1_isr()
{
	if(x == 2){     // Define o tempo desligado, Ciclo para x: 1=10.8ms, 2=16.4ms, 3=22.8ms, 4=26ms
      if( LIG_DESL_SERVOS == 1 ){	
      	CommandServos();
	  }
	  x=0;	
    }
	else{
      x++;
    }
}
//****************************************************************************

//****************************************************************************
#int_TIMER0
void TIMER0_isr()
{
	  set_timer0(120);
	  rotina_1_ms();
}
//****************************************************************************
